/*
 * Project: MoleCuilder
 * Description: creates and alters molecular systems
 * Copyright (C)  2014 Frederik Heber. All rights reserved.
 * 
 *
 *   This file is part of MoleCuilder.
 *
 *    MoleCuilder is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    MoleCuilder is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MoleCuilder.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * TranslateAction.cpp
 *
 *  Created on: Dec 14, 2014
 *      Author: heber
 */

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

//#include "CodePatterns/MemDebug.hpp"

#include "CodePatterns/Chronos.hpp"

#include "Actions/MoleculeAction/TranslateAction.hpp"

#include "Actions/SelectionAction/Atoms/PushAtomsAction.hpp"
#include "Actions/SelectionAction/Atoms/PopAtomsAction.hpp"
#include "Actions/SelectionAction/Atoms/AllAtomsAction.hpp"
#include "Actions/AtomAction/TranslateAction.hpp"
#include "Actions/ActionQueue.hpp"
#include "Actions/ActionSequence.hpp"

#include "Descriptors/AtomDescriptor.hpp"

using namespace MoleCuilder;

// and construct the stuff
#include "TranslateAction.def"
#include "MakroAction_impl_pre.hpp"
/** =========== define the function ====================== */

// static instances
ActionSequence MoleculeTranslateAction::prototype_actions;
bool MoleculeTranslateAction::isPrepared = false;

void MoleculeTranslateAction::prepare(ActionRegistry &AR)
{
  // we simply have to select all the molecule's atoms and use the respective
  // translate Action
  prototype_actions.addAction(AR.getActionByName(std::string("push-atom-selection")).clone());
  prototype_actions.addAction(AR.getActionByName(std::string("select-molecules-atoms")).clone());
  prototype_actions.addAction(AR.getActionByName(std::string("translate-atoms")).clone());
  prototype_actions.addAction(AR.getActionByName(std::string("pop-atom-selection")).clone());
  isPrepared = true;
}

void MoleculeTranslateAction::unprepare(ActionRegistry &AR)
{
  // empty sequence
  Action *actionremove = NULL;
  while ((actionremove = prototype_actions.removeLastAction()) != NULL)
    delete actionremove;
  isPrepared = false;
}

ActionState::ptr MoleculeTranslateAction::performCall(){
  ActionState::ptr state(MakroAction::performCall());

  return state;
}

ActionState::ptr MoleculeTranslateAction::performUndo(ActionState::ptr _state) {
  ActionState::ptr state(MakroAction::performUndo(_state));

  return state;
}

ActionState::ptr MoleculeTranslateAction::performRedo(ActionState::ptr _state){
  ActionState::ptr state(MakroAction::performRedo(_state));

  return state;
}

bool MoleculeTranslateAction::canUndo(){
  return true;
}

bool MoleculeTranslateAction::shouldUndo(){
  return true;
}
